
#include "x.h"
#include "xt.h"
#include "xpex.h"
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "book_utils.h"
#include "externs.h"
#include "defs.h"
#include "globals.h"
#include "color.h"

Display *dpy;
Window   window;

XtInputId specInput, matInput;

PEXRenderer           renderer;
PEXRendererAttributes rattrs;

PEXStructure top_struct;
Atom         delete_window_atom;
Widget       toplevel, canvas;

/* Callback function to quit the program. */
void
quit( w, client_data, call_data )
Widget     w;
caddr_t    client_data;
XtPointer *call_data;
{
#if 0
    XClientMessageEvent        *cm = &call_data->event->xclient;

    if ( cm->data.l[0] == delete_window_atom ) {
        XtDestroyWidget( toplevel );
        exit( 0 );
    }
#endif
}


/* Callback for Expose events. */
void
expose( w, client_data, event )
Widget    w;
XtPointer    client_data;
XEvent    *event;
{
    /* Redraw when last Expose event received. */
    if ( event->xexpose.count == 0 )
        redraw();
}


void
set_depth_cue_color( cbs )
char *cbs;
{
    char type[3];
    float val1, val2, val3;
    String string = (String) malloc(26);
    extern Widget depthCueColorLabel;
    extern PEXDepthCueEntry depth_cue_entry;
    extern PEXRendererAttributes rattrs;

    XFlush( dpy );

    sscanf( cbs, "%s %f %f %f\n", &type, &val1, &val2, &val3 );
    if( !strcmp( type, "RGB" ) ) {
        depth_cue_entry.color.type = PEXColorTypeRGB;
        depth_cue_entry.color.value.rgb.red = val1;
        depth_cue_entry.color.value.rgb.green = val2;
        depth_cue_entry.color.value.rgb.blue = val3;
        sprintf( string, "R: %.2f G: %.2f B: %.2f", val1, val2, val3 );
        XtVaSetValues( depthCueColorLabel, XtNlabel, string, NULL );
    }
    else if( !strcmp( type, "HSV" ) ) {
        depth_cue_entry.color.type = PEXColorTypeHSV;
        depth_cue_entry.color.value.hsv.hue = val1;
        depth_cue_entry.color.value.hsv.saturation = val2;
        depth_cue_entry.color.value.hsv.value = val3;
        sprintf( string, "H: %.2f S: %.2f V: %.2f", val1, val2, val3 );
        XtVaSetValues( depthCueColorLabel, XtNlabel, string, NULL );
        if( !colorTypeHSV ) {
            HSV hsv_values;
            RGB rgb_values;

            hsv_values.h = val1;
            hsv_values.s = val2;
            hsv_values.v = val3;
            rgb_values = HSVToRGB( hsv_values );
            depth_cue_entry.color.value.rgb.red   =
                (float)rgb_values.r / 65535.0;
            depth_cue_entry.color.value.rgb.green =
                (float)rgb_values.g / 65535.0;
            depth_cue_entry.color.value.rgb.blue  =
                (float)rgb_values.b / 65535.0;
        }
    }
    else if( !strcmp( type, "HLS" ) ) {
        depth_cue_entry.color.type = PEXColorTypeHLS;
        depth_cue_entry.color.value.hls.hue = val1;
        depth_cue_entry.color.value.hls.lightness = val2;
        depth_cue_entry.color.value.hls.saturation = val3;
        sprintf( string, "H: %.2f L: %.2f S: %.2f", val1, val2, val3 );
        XtVaSetValues( depthCueColorLabel, XtNlabel, string, NULL );
        if( !colorTypeHLS ) {
            HLS hls_values;
            RGB rgb_values;

            hls_values.h = val1;
            hls_values.l = val2;
            hls_values.s = val3;
            rgb_values = HLSToRGB( hls_values );
            depth_cue_entry.color.value.rgb.red =
                (float)rgb_values.r / 65535.0;
            depth_cue_entry.color.value.rgb.green =
                (float)rgb_values.g / 65535.0;
            depth_cue_entry.color.value.rgb.blue =
                (float)rgb_values.b / 65535.0;
        }
    }
    else if( !strcmp( type, "CIE" ) ) {
        depth_cue_entry.color.type = PEXColorTypeCIE;
        depth_cue_entry.color.value.cie.x = val1;
        depth_cue_entry.color.value.cie.y = val2;
        depth_cue_entry.color.value.cie.z = val3;
        sprintf( string, "X: %.2f Y: %.2f Z: %.2f", val1, val2, val3 );
        XtVaSetValues( depthCueColorLabel, XtNlabel, string, NULL );
        if( !colorTypeCIE ) {
            CIE cie_values;
            RGB rgb_values;

            cie_values.X = val1;
            cie_values.Y = val2;
            cie_values.Z = val3;
            rgb_values = CIEToRGB( cie_values );
            depth_cue_entry.color.value.rgb.red =
                (float)rgb_values.r / 256.0;
            depth_cue_entry.color.value.rgb.green =
                (float)rgb_values.g / 256.0;
            depth_cue_entry.color.value.rgb.blue =
                (float)rgb_values.b / 256.0;
        }
    }

    XtRemoveInput( specInput );
    PEXSetTableEntries( dpy, rattrs.depth_cue_table, 1, 1,
        PEXLUTDepthCue, &depth_cue_entry );
    redraw();
}


void
set_depth_cue( widget, parameter, cbs )
Widget    widget;
caddr_t   parameter;
XtPointer cbs;
{
    extern Widget depthCueBackPlLabel, depthCueFrontPlLabel,
                  depthCueBackScLabel, depthCueFrontScLabel;
    String string;
    float per = *( float* )cbs;
    float top = ( float )per;
    extern PEXDepthCueEntry depth_cue_entry;
    extern PEXRendererAttributes rattrs;

    string = (String) malloc(10);
    sprintf( string, "%.2f", top );

    switch ( (int)parameter ) {
    case BACK_PLANE:
        depth_cue_entry.back_plane = top;
        XtVaSetValues( depthCueBackPlLabel, XtNlabel, string, NULL );
        break;
    case FRONT_PLANE:
        depth_cue_entry.front_plane = top;
        XtVaSetValues( depthCueFrontPlLabel, XtNlabel, string, NULL );
        break;
    case BACK_SCALING:
        depth_cue_entry.back_scaling = top;
        XtVaSetValues( depthCueBackScLabel, XtNlabel, string, NULL );
        break;
    case FRONT_SCALING:
        depth_cue_entry.front_scaling = top;
        XtVaSetValues( depthCueFrontScLabel, XtNlabel, string, NULL );
        break;
    }

    PEXSetTableEntries( dpy, rattrs.depth_cue_table, 1, 1,
        PEXLUTDepthCue, &depth_cue_entry );
    redraw();
}


void editDepthCueColor( widget, call_data, client_data )
Widget    widget;
XtPointer call_data, client_data;
{
    extern XtAppContext app;
    extern char *getenv();
    extern PEXDepthCueEntry depth_cue_entry;
    extern void get_depth_cue_reply();

    if( getenv( "COLOR_EDITOR" ) ) {
        char *program, *args;

        specInput = XtAppAddInput( app, fileno(stdin), (XtPointer)XtInputReadMask, get_depth_cue_reply, NULL );

        program = (char *) malloc( strlen(getenv( "COLOR_EDITOR" )) + 25 );
        strcpy( program, getenv( "COLOR_EDITOR" ) );
        args = (char *) malloc(25);
        sprintf( args, " RGB %.2f %.2f %.2f", depth_cue_entry.color.value.rgb.red, depth_cue_entry.color.value.rgb.green, depth_cue_entry.color.value.rgb.blue );
        strcat( program, args );
        xs_talkto( program );
    }
    else {
        fprintf( stderr, "Environment variable COLOR_EDITOR, undefined!!!\n\n");
        XtDestroyApplicationContext( app );
        exit( 0 );
    }
}


void get_depth_cue_reply(client_data, fid, id)
XtPointer    client_data;
int         *fid;
XtInputId   *id;
{
    char       buf[BUFSIZ];
    int        nbytes, i;

    /* Get all pending input */
    nbytes = read(*fid, buf, BUFSIZ);

    /*
     * Null terminate the string at the first newline,
     * or at the end of the bytes read.
     */
    if (nbytes && buf[0] != '\n') {
        for( i= 0; i< nbytes; i++ )
            if(buf[i] == '\n')
                buf[i] = '\0';
        buf[nbytes] = '\0';
        set_depth_cue_color( buf );
    }
}
