
#include "x.h"
#include "xt.h"
#include "globals.h"
#include "defs.h"
#include "externs.h"
#include <stdio.h>
#ifdef XPM
#include <X11/xpm.h>
#endif

Widget depthCueBackPlLabel, depthCueFrontPlLabel,
       depthCueBackScLabel, depthCueFrontScLabel,
       depthCueColorLabel;

#ifdef XPM
void errorMessageXPM(ErrorStatus, tag)
int ErrorStatus;
char *tag;
{
    char *error = NULL;
    char *warning = NULL;

    switch (ErrorStatus) {
    case XpmSuccess:
        return;
    case XpmColorError:
        warning = "Could not parse or alloc requested color";
        break;
    case XpmOpenFailed:
        error = "Cannot open file";
        break;
    case XpmFileInvalid:
        error = "Invalid XPM file";
        break;
    case XpmNoMemory:
        error = "Not enough memory";
        break;
    case XpmColorFailed:
        error = "Failed to parse or alloc some color";
        break;
    }

    if (warning)
        printf("%s Xpm Warning: %s.\n", tag, warning);

    if (error) {
        printf("%s Xpm Error: %s.\n", tag, error);
/*      exit( -1 ); */
    }
}
#endif


void
create_gui( depth, vendor )
int depth;
char *vendor;
{
    int status, i;
    float topOfThumb;
    XtArgVal *l_topOfThumb;
    String string = (String) malloc(30);
    Dimension width, height, widthScr;

    extern PEXDepthCueEntry depth_cue_entry;
    extern PEXColor colorDepthCue;

    extern Widget toplevel;
    Widget quit, form,
           depthCueForm,
           depthCueLabel,
           depthCueBackPl, depthCueFrontPl,
           depthCueBackPlScroll, depthCueFrontPlScroll,
           depthCueBackSc, depthCueFrontSc,
           depthCueBackScScroll, depthCueFrontScScroll,
           depthCueColor;

#ifdef XPM
    Pixmap backPixmap, pix;
    XpmAttributes attributes;

    if( depth == 8 || (strcmp( vendor, "International Business Machines" )) ) {
        attributes.valuemask = XpmReturnPixels;
        status = XpmReadFileToPixmap( XtDisplay(toplevel),
            DefaultRootWindow(XtDisplay(toplevel)),
            "pixmaps/background.xpm", &backPixmap, &pix, &attributes );
        errorMessageXPM( status, "Background" );
    }
#endif

    /* Create the widgets. */
    if( depth == 8 || (strcmp( vendor, "International Business Machines" )) )
    form = XtVaCreateManagedWidget( "form", formWidgetClass, toplevel,
#ifdef XPM
        XtNbackgroundPixmap, backPixmap,
#endif
        NULL );
    else
    form = XtVaCreateManagedWidget( "form", formWidgetClass, toplevel,
        NULL );

    canvas = XtVaCreateManagedWidget( "canvas", formWidgetClass, form,
        XtNwidth, 210,
        XtNheight, 210,
        NULL );
    XtAddEventHandler( canvas, ExposureMask, FALSE, expose, NULL );

    /* The depthCue properties panel. */
    if( depth == 8 || (strcmp( vendor, "International Business Machines" )) )
    depthCueForm = XtVaCreateManagedWidget( "depthCueForm", formWidgetClass,
        form,
        XtNfromHoriz, canvas,
        XtNborderWidth, 0,
#ifdef XPM
        XtNbackgroundPixmap, backPixmap,
#endif
        NULL );
    else
    depthCueForm = XtVaCreateManagedWidget( "depthCueForm", formWidgetClass,
        form,
        XtNfromHoriz, canvas,
        XtNborderWidth, 0,
        NULL );

    depthCueLabel = XtVaCreateManagedWidget( "Label", labelWidgetClass,
        depthCueForm,
        XtNlabel, "Depth Cue:",
        XtNborderWidth, 0,
        NULL );

    depthCueBackPl = XtVaCreateManagedWidget( "BackPl", labelWidgetClass,
        depthCueForm,
        XtNlabel, "Back plane:",
        XtNwidth, 122,
        XtNfromVert, depthCueLabel,
        NULL );
    XtVaGetValues( depthCueBackPl, XtNheight, &height, NULL );

    topOfThumb = depth_cue_entry.back_plane;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    depthCueBackPlScroll = XtVaCreateManagedWidget( "BackPlScroll",
        scrollbarWidgetClass,
        depthCueForm,
        XtNfromVert, depthCueLabel,
        XtNfromHoriz, depthCueBackPl,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtAddCallback( depthCueBackPlScroll, XtNjumpProc, set_depth_cue, (XtPointer)BACK_PLANE );

    sprintf( string, "%.2f", depth_cue_entry.back_plane );
    depthCueBackPlLabel = XtVaCreateManagedWidget( "BackPlLabel",
        labelWidgetClass,
        depthCueForm,
        XtNfromVert, depthCueLabel,
        XtNfromHoriz, depthCueBackPlScroll,
        XtNlabel, string,
        NULL );

    depthCueFrontPl = XtVaCreateManagedWidget( "FrontPl",
        labelWidgetClass,
        depthCueForm,
        XtNlabel, "Front plane:",
        XtNwidth, 122,
        XtNfromVert, depthCueBackPl,
        NULL );

    topOfThumb = depth_cue_entry.front_plane;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    depthCueFrontPlScroll = XtVaCreateManagedWidget( "FrontPlScroll",
        scrollbarWidgetClass,
        depthCueForm,
        XtNfromVert, depthCueBackPl,
        XtNfromHoriz, depthCueFrontPl,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtAddCallback( depthCueFrontPlScroll, XtNjumpProc, set_depth_cue, (XtPointer)FRONT_PLANE );

    sprintf( string, "%.2f", depth_cue_entry.front_plane );
    depthCueFrontPlLabel = XtVaCreateManagedWidget( "FrontPlLabel",
        labelWidgetClass,
        depthCueForm,
        XtNfromVert, depthCueBackPl,
        XtNfromHoriz, depthCueFrontPlScroll,
        XtNlabel, string,
        NULL );

    depthCueBackSc = XtVaCreateManagedWidget( "BackSc",
        labelWidgetClass,
        depthCueForm,
        XtNlabel, "Back scaling:",
        XtNwidth, 122,
        XtNfromVert, depthCueFrontPl,
        NULL );

    topOfThumb = depth_cue_entry.back_scaling;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    depthCueBackScScroll = XtVaCreateManagedWidget( "BackScScroll",
        scrollbarWidgetClass,
        depthCueForm,
        XtNfromVert, depthCueFrontPl,
        XtNfromHoriz, depthCueBackSc,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtAddCallback( depthCueBackScScroll, XtNjumpProc, set_depth_cue, (XtPointer)BACK_SCALING );

    sprintf( string, "%.2f", depth_cue_entry.back_scaling );
    depthCueBackScLabel = XtVaCreateManagedWidget( "BackScLabel",
        labelWidgetClass,
        depthCueForm,
        XtNfromVert, depthCueFrontPl,
        XtNfromHoriz, depthCueBackScScroll,
        XtNlabel, string,
        NULL );

    depthCueFrontSc = XtVaCreateManagedWidget( "FrontSc",
        labelWidgetClass,
        depthCueForm,
        XtNlabel, "Front Scaling:",
        XtNwidth, 122,
        XtNfromVert, depthCueBackSc,
        NULL );

    topOfThumb = depth_cue_entry.front_scaling;
    l_topOfThumb = (XtArgVal *) &topOfThumb;
    depthCueFrontScScroll = XtVaCreateManagedWidget( "FrontScScroll",
        scrollbarWidgetClass,
        depthCueForm,
        XtNfromVert, depthCueBackSc,
        XtNfromHoriz, depthCueFrontSc,
        XtNtopOfThumb, *l_topOfThumb,
        XtNheight, height,
        NULL );
    XtVaGetValues( depthCueFrontScScroll, XtNwidth, &widthScr, NULL );
    XtAddCallback( depthCueFrontScScroll, XtNjumpProc, set_depth_cue, (XtPointer)FRONT_SCALING );

    sprintf( string, "%.2f", depth_cue_entry.front_scaling );
    depthCueFrontScLabel = XtVaCreateManagedWidget( "FrontScLabel",
        labelWidgetClass,
        depthCueForm,
        XtNfromVert, depthCueBackSc,
        XtNfromHoriz, depthCueFrontScScroll,
        XtNlabel, string,
        NULL );
    XtVaGetValues( depthCueFrontScLabel, XtNwidth, &width, NULL );

    depthCueColor = XtVaCreateManagedWidget( "Color", commandWidgetClass,
        depthCueForm,
        XtNfromVert, depthCueFrontScScroll,
        XtNlabel, "Depth Cue Color:",
        XtNwidth, 122,
        NULL );
    XtAddCallback( depthCueColor, XtNcallback, editDepthCueColor, NULL );

    sprintf( string, "R: %.2f G: %.2f B: %.2f", depth_cue_entry.color.value.rgb.red, depth_cue_entry.color.value.rgb.green, depth_cue_entry.color.value.rgb.blue );
    depthCueColorLabel = XtVaCreateManagedWidget( "ColorLabel", labelWidgetClass,
        depthCueForm,
        XtNfromVert, depthCueFrontScScroll,
        XtNfromHoriz, depthCueColor,
        XtNlabel, string,
        XtNwidth, 8+width+widthScr,
        NULL );

    quit = XtVaCreateManagedWidget( "quit", commandWidgetClass, form,
        XtNfromVert, canvas,
        NULL );
    XtAddCallback( quit, XtNcallback, finalQuit, NULL );
}

void
finalQuit( w, client_data, call_data )
Widget w;
XtPointer client_data, call_data;
{
    extern XtAppContext app;
    extern void printDepthCue();

    printDepthCue();
    XtDestroyApplicationContext( app );
    (void) exit (0);
}
