
#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#ifndef USE_ATHENA
#include "xm.h"
#else
#include "xt.h"
#endif

#include "menus.h"

void
create_menus( title, menu, menulist, nitems )
char        *title;
Widget       menu;
menu_struct *menulist;
int          nitems;
{
    int        i;
    WidgetList buttons;
    int        separators = 0;
    /*
     * Allocate a widget list to hold all 
     * button widgets.
     */
    buttons = (WidgetList)malloc( nitems * sizeof(Widget) );
    /*
     * If a title is given, create Label and Separator widgets.
     */
    if( title ) {
        XtCreateManagedWidget( title,
#ifndef USE_ATHENA
            xmLabelWidgetClass,
#else
            labelWidgetClass,
#endif
            menu, NULL, 0 );
        XtCreateManagedWidget( "separator",
#ifndef USE_ATHENA
            xmSeparatorWidgetClass,
#else
            smeLineObjectClass,
#endif
            menu, NULL, 0 );
    }
    /*
     * Create an entry for each item in the menu.
     */
    for( i=0; i<nitems; i++ ) {
        /* 
         * A NULL name represents a separator.
         */
        if( menulist[i].name == (char *)NULL ) {
            XtCreateManagedWidget( "separator",
#ifndef USE_ATHENA
                xmSeparatorWidgetClass,
#else
                smeLineObjectClass,
#endif
                menu, NULL, 0 );
            separators++; /* Count how many entries aren't buttons */
        }
        /*
         * If there is a name and a callback, create a "normal"
         * menu entry and register the callback function.
         */
        else if( menulist[i].func ) {
            buttons[i-separators] = XtCreateWidget( menulist[i].name,
#ifndef USE_ATHENA
                xmPushButtonWidgetClass,
#else
                smeBSBObjectClass,
#endif
                menu, NULL, 0 );
            XtAddCallback( buttons[i-separators],
#ifndef USE_ATHENA
                XmNactivateCallback, (XtCallbackProc)menulist[i].func,
#else
                XtNcallback, (XtCallbackProc)menulist[i].func,
#endif
                menulist[i].data );
            if( menulist[i].func1 )
                XtAddCallback( buttons[i-separators],
#ifndef USE_ATHENA
                    XmNactivateCallback, (XtCallbackProc)menulist[i].func1,
#else
                    XtNcallback, (XtCallbackProc)menulist[i].func1,
#endif
                    NULL );
        }
        /*
         * If there is a name, but no callback function, the entry
         * must be a label, unless there is a submenu.
         */
        else if( !menulist[i].sub_menu ) {
            buttons[i-separators] = XtCreateWidget( menulist[i].name,
#ifndef USE_ATHENA
                xmLabelWidgetClass,
#else
                labelWidgetClass,
#endif
                menu, NULL, 0 );
        }
        /*
         * If we got here, the entry must be a submenu.
         * Create a pulldown menu pane and an XmCascadeButton
         * widget. Attach the menu pane and make a recursive call
         * to create the entries in the submenu.
         */
        else {
            Widget sub_menu;
#ifndef USE_ATHENA
            Arg wargs[1];
            sub_menu = XmCreatePulldownMenu( menu,
                menulist[i].sub_menu_title,
                NULL, 0 );
            XtSetArg( wargs[0], XmNsubMenuId, sub_menu );
            buttons[i-separators] = XtCreateWidget( menulist[i].name,
                xmCascadeButtonWidgetClass, menu,
                wargs, 1 );
#else
            Widget sub_menu_entry, sub_menu_button;
            sub_menu_button = XtVaCreateManagedWidget( "sub_menu_button",
                menuButtonWidgetClass, menu,
                XtNlabel, menulist[i].name,
                XtNmenuName, "sub_menu",
                NULL );
            sub_menu = XtVaCreatePopupShell( "sub_menu",
                simpleMenuWidgetClass, sub_menu_button,
                NULL );
            buttons[i-separators] = XtCreateWidget( menulist[i].name,
                smeBSBObjectClass, sub_menu,
                NULL, 0 );
#endif
            create_menus( menulist[i].sub_menu_title, sub_menu,
                menulist[i].sub_menu, menulist[i].n_sub_items );
        }
    }
    /*
     * Manage all button widgets. Menu panes are not managed.
     */
    XtManageChildren( buttons, nitems - separators );
}
